// Lucas Keyes
// main code for SpriteImg_ManagerObject

#include "SpriteImg_ManagerObject.h"


SpriteImg_ManagerObject::SpriteImg_ManagerObject() {
	// constructor, sets the array to null values

	for (int index = 0; index < MAX_SPRITES ; index++ )
		sprite_array[index] = NULL;

	size = 0;

}

SpriteImg_ManagerObject::~SpriteImg_ManagerObject() {
	// destructor, releases all the surfaces and deletes
	// all the array values

	Release_All();		// release all surfaces
	
	for (int index = 0; index < MAX_SPRITES; index++)
		delete sprite_array[index];
}

void SpriteImg_ManagerObject::draw(int index, int frame, 
								   int x, int y,
								   LPDIRECTDRAWSURFACE destination) {
	
	if ( index < size ) {	// test if index is in the array's range

		if (sprite_array[index] != NULL)	// make sure the location
											// contains a valid sprite
											// object

			// use the sprite's draw function to draw to surface
			sprite_array[index]->draw(frame,x,y,destination);

	}

}

BOOL SpriteImg_ManagerObject::load(int index, SpriteBmp_Object *image,
								   int frame,
								   int x, int y ) {

	// test if index can be put in the array
	if (index < size )	{

		// make sure array location contains a valid object
		if (sprite_array[index] != NULL) 
		 return	sprite_array[index]->load(image,frame,x,y);
			
	}

	return false;
}


bool SpriteImg_ManagerObject::create(int index,int frames,
									 int w, int h,
									 bool clipped) {

	// this method creates a new SpriteImg object in the array
	// at the location index

	if (index < MAX_SPRITES) {
		// index is smaller than the maximum amount of sprites

		if ( sprite_array[index] == NULL ) {
			// array location is currently empty

			if ( clipped )
				// make a clipped sprite, rather than a normal sprite
				sprite_array[index] = new ClipSprtImg_Object(frames,
															w, h);
			else
				// make a normal sprite
				sprite_array[index] = new SpriteImg_Object( frames,
				                                           w, h );

			if (index > size - 1)
				size = index + 1;	// keeps size equal to last index
									// value plus 1.

		}

		return true;
	}

	return false;
}

int SpriteImg_ManagerObject::frame_count(int index) {

	if (index < size ) {
		// location is valid index

		if (sprite_array[index] != NULL) {
			// location contains valid sprite
			return sprite_array[index]->frame_count();

		}
	}

	return 0;
}

SpriteImg_Object *SpriteImg_ManagerObject::getPtr(int index) {
	// method checks if the index is in a valid location and
	// returns the memory location

	if (index < MAX_SPRITES )	// if index is at a valid location
		return sprite_array[index];		// return the memory location
										// of the SpriteImg
	else
		return NULL;			// return NULL otherwise
}

int SpriteImg_ManagerObject::height(int index) {
	// this method checks to see if the location at the array
	// index is a valid Sprite Image and returns the height

	if (index < size )	// if index is less than the size of the array
		if ( sprite_array[index] != NULL ) 
			// if location does not contain a null reference.
			return sprite_array[index]->height();
		

	
	return 0;

}

void SpriteImg_ManagerObject::Release(int index) {
	// this method checks to see if the location at the array
	// index is a valid Sprite Image and releases the Sprite's
	// image surface

	if (index < size ) // if index is less than the size of the array
		if ( sprite_array[index] != NULL )
			// if location does not contain a null reference
			sprite_array[index]->Release();

}


void SpriteImg_ManagerObject::Release_All() {
	// this releases all the sprite image surfaces contained
	// in the sprite_array

	for (int index = 0; index < size; index++ )
		Release(index);

}

void SpriteImg_ManagerObject::Restore(int index) {
	// this method checks to see if the location at the array
	// index is a valid Sprite Image and restores the Sprite's
	// image surface

		if (index < size ) // if index is less than the size of the array
		if ( sprite_array[index] != NULL )
			// if location does not contain a null reference
			sprite_array[index]->Release();
}

void SpriteImg_ManagerObject::Restore_All() {
	// this releases all the sprite image surfaces contained
	// in the sprite_array

	for (int index = 0; index < size; index++ )
		Restore(index);
}

int SpriteImg_ManagerObject::width(int index) {
	// this method checks to see if the location at the array
	// index is a valid Sprite Image and returns the height

	if (index < size )	// if index is less than the size of the array
		if ( sprite_array[index] != NULL )
			// if location does not contain a null reference.
			return sprite_array[index]->width();

	
	return 0;

}

