// bmp.cpp: member functions for the bmp file reader

// updated by Lucas Keyes
// January 17, 2002

#include "bmp.h"    // header file

// constructors and destructors
BmpFile_Object::BmpFile_Object(){    // constructor
    b_Image = 0;	// memory not yet allocated
}

BmpFile_Object::~BmpFile_Object() { // destructor
    delete[]b_Image;	// reclaim memory from image data
}

// member functions

BOOL BmpFile_Object::load(char *filename) { // load bmp from file
    HANDLE hfile;	// input file handle
    DWORD actualRead;	// number of bytes actually read
    int image_size;	// size of image (width * height)
    BOOL OK = TRUE;	// no error has occurred yet

    // open input file for reading
    hfile = CreateFile(filename, GENERIC_READ,FILE_SHARE_READ,
						(LPSECURITY_ATTRIBUTES)NULL,OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,(HANDLE)NULL);
    // bail if file could not be opened
    if (hfile == INVALID_HANDLE_VALUE) return FALSE;
    // read the header and info structures
    OK = ReadFile(hfile,&BmpFileHead,sizeof(BmpFileHead),
                  &actualRead,NULL);
    if (OK) OK = ReadFile(hfile,&BmpFileInfo,sizeof(BmpFileInfo),
		          &actualRead,NULL);

    // make sure the bmp is 8-bit color
    if (OK) OK = BmpFileInfo.biBitCount == COLOR_DEPTH;

    // get the bmp palette
    if (OK) OK = ReadFile(hfile,rgbPalette,sizeof(rgbPalette),
                          &actualRead,NULL);

    // bail out is something went wrong
    if (!OK) {CloseHandle(hfile); return FALSE; }

    // allocate memory for image data
    image_size = BmpFileInfo.biWidth * BmpFileInfo.biHeight;
    if (b_Image) delete[]b_Image;	// dispose of any old space
    b_Image = new BYTE[image_size];     // allocate new space
    if (!b_Image) {			// bail if alloc failed
       CloseHandle(hfile); return FALSE;
    }

    // round up line width to next multiple of 4
    int width = (BmpFileInfo.biWidth + 3)&~3;
    // read bmp image
    int i = 0;  // counter
    BYTE trash[4];  // to hold the trash at the end of each line
    int remainder = width - BmpFileInfo.biWidth;   // width of trash

    while (OK&&i<BmpFileInfo.biHeight) {
        // read data
        OK = OK&&ReadFile(hfile, (BYTE*) (b_Image+i*BmpFileInfo.biWidth),
                          BmpFileInfo.biWidth,&actualRead,NULL);
        // read trash at end of line
        OK = OK&&ReadFile(hfile,trash,remainder,&actualRead,NULL);
        i++;  // next line
    }

    if (!OK) delete[]b_Image;	// clean up if failed
    // close up and exit
    CloseHandle(hfile);
    return OK;
}  // end load

BOOL BmpFile_Object::draw(LPDIRECTDRAWSURFACE surface) {
    DDSURFACEDESC ddsd;    // direct draw surface descriptor
    BYTE *dest, *src;	   // destination and source pointers
    int src_width;         // width of source

    // init surface descriptor
    memset(&ddsd,0,sizeof(DDSURFACEDESC));
    ddsd.dwSize = sizeof(ddsd);
    
    // lock down surface
    if (FAILED(surface->Lock(NULL,&ddsd,DDLOCK_WAIT,NULL)))
       return FALSE;

    // calculate addresses and width
    dest = (BYTE*)ddsd.lpSurface;	// destination
    src = b_Image + ((BmpFileInfo.biHeight -1) * BmpFileInfo.biWidth);
    // trim bmp if too wide
    if (BmpFileInfo.biWidth>ddsd.lPitch) src_width = ddsd.lPitch;
    else src_width = BmpFileInfo.biWidth;
    
    // move data to surface
    for (int i = 0; i<BmpFileInfo.biHeight; i++) {
        memcpy(dest,src,src_width);
        dest+=ddsd.lPitch; src-=src_width;
    }

    // clean up and return
    surface->Unlock(NULL);
    return TRUE;
} // end draw

BOOL BmpFile_Object::setpalette(LPDIRECTDRAWPALETTE palette){
  PALETTEENTRY pe[COLORS]; //intermediate palette
  //construct pe[]
  for(int i=0; i<COLORS; i++){ //for each palette entry
    pe[i].peRed=rgbPalette[i].rgbRed; //set red
    pe[i].peGreen=rgbPalette[i].rgbGreen; //set green
    pe[i].peBlue=rgbPalette[i].rgbBlue; //set blue
  }
  //load direct draw palette to surface
  palette->SetEntries(NULL,0,COLORS,pe);
  return TRUE;
}

    