// bmp.h: header file for the bmp file reader.
// the BmpFile_Object class stores all the information of the bitmap in raw data.
// it is responsible for reading in a bitmap from a file, and drawing the raw
// data to a DirectDraw surface.  It can also set the palette for the surface based
// on the palette in the file.

// Updated by Lucas Keyes
// January 17, 2002

#include <windows.h>
#include <windowsx.h>
#include <ddraw.h>

#include "defines.h"  // holds all the constant declarations for the program

#ifndef __bmp_h__     // ensures that there is only one instance of bmp.h
#define __bmp_h__     // declared, preventing a run-time error

class BmpFile_Object{	// bmp file input class
    protected:				// protected variables allow inheritance for derived classes
       BITMAPFILEHEADER BmpFileHead;	// bmp header
       BITMAPINFOHEADER BmpFileInfo;    // bmp file information
       RGBQUAD rgbPalette[COLORS]; 	// the palette for the bitmap
       BYTE *b_Image;			// pointer to the raw data of the bitmap (dynamically allocated)

    public:
       BmpFile_Object();		// constructor
       ~BmpFile_Object();		// destructor
       BOOL load(char *filename);	// loads bitmap from file, returns TRUE if success
       BOOL draw(LPDIRECTDRAWSURFACE surface);	// copies raw data onto a DirectDraw surface
       BOOL setpalette(LPDIRECTDRAWPALETTE pal);  // sets the palette based on bitmap's palette


};

#endif