// Lucas Keyes
// functions for DirectX Manager object

#define DIRECTINPUT_VERSION 0x0700 // needed to allow DirectX 7 input
#include "defines.h"

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 

#define WIN32_LEAN_AND_MEAN  


#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <io.h>
#include <fcntl.h>


#include "dxmanager.h"

DirectXManagerObject::DirectXManagerObject() {
	// initializes all data pointers in the object
	// to null;

	lpdirectinputobj = NULL;
	lpdirectsoundobj = NULL;
	lpdirectdrawobj = NULL;

	lpddsprimary = lpddsecondary = NULL;

	lpddsprimarypalette = lpddsecondarypalette = NULL;

	lpdinputkey = NULL;


}



LPDIRECTDRAWPALETTE DirectXManagerObject::CreatePalette(LPDIRECTDRAWSURFACE surface){
//create surface palette

  PALETTEENTRY pe[COLORS]; //new palette
  LPDIRECTDRAWPALETTE lpDDPalette; //direct draw palette
  //construct pe[], set to black
  for(int i=0; i<COLORS; i++)
    pe[i].peRed=pe[i].peGreen=pe[i].peBlue=0;
  //create direct draw palette
  if(FAILED(lpdirectdrawobj->
  CreatePalette(DDPCAPS_8BIT,pe,&lpDDPalette,NULL)))
    return NULL;
  //load direct draw palette to surface
  surface->SetPalette(lpDDPalette);
  return lpDDPalette;
} //CreatePalette


int DirectXManagerObject::createdirectdraw(int width, int height, int bitsperpxl, HWND window_handle) {
	// this function creates a fullscreen DirectDraw object


	// create Direct Draw Object
	if (DirectDrawCreate(NULL,&lpdirectdrawobj,NULL) != DD_OK)
		return(0);
	
	// set cooperation level
	lpdirectdrawobj->SetCooperativeLevel(window_handle,
						DDSCL_FULLSCREEN|DDSCL_EXCLUSIVE|
						DDSCL_ALLOWREBOOT|DDSCL_ALLOWMODEX);
	
	// set the video mode according to the width, height, and bpp
	if (lpdirectdrawobj->SetDisplayMode(width,height,bitsperpxl) != DD_OK)
		return(0);

	// set up the DirectDraw clipper

	// create it
	if (FAILED(lpdirectdrawobj->CreateClipper(NULL,&lpddsclipper,NULL)))
		return(0);

	// attach it to the window handle
	if (FAILED(lpddsclipper->SetHWnd(NULL,window_handle)))
		return(0);

	return(1);
}


int DirectXManagerObject::createsurfaces() {
	// this function sets up a primary surface with a secondary
	// backbuffer that allows flipping.

	DDSURFACEDESC ddsd;
	DDSCAPS ddscaps;

	memset(&ddsd,0,sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);	// set size of surface

	// set capabilities and backbuffer in the controlling flags
	ddsd.dwFlags = DDSD_CAPS|DDSD_BACKBUFFERCOUNT;

	// set capabilities to primary, complex, and flippable
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE|DDSCAPS_FLIP|
							DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1;	// set number of sec surfaces to one

	// create the primary surface
	if (lpdirectdrawobj->CreateSurface(&ddsd,&lpddsprimary,NULL) != DD_OK)
		return(0);
	//create its palette
	lpddsprimarypalette=CreatePalette(lpddsprimary);

	// get the secondary surface
	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
	lpddsprimary->GetAttachedSurface(&ddscaps,&lpddsecondary);
	//create its palette
	lpddsecondarypalette=CreatePalette(lpddsecondary);

	
	return(1);

}

void DirectXManagerObject::clearsecsurface() {

	DDBLTFX ddbltfx;	// blitter description

	// clear out description variable and set size
	memset(&ddbltfx,0,sizeof(DDBLTFX));
	ddbltfx.dwSize = sizeof(DDBLTFX);

	// set fill color to BLACK
	ddbltfx.dwFillColor = 0;

	// blit color to whole secondary surface
	lpddsecondary->Blt(NULL,
		NULL,
		NULL,DDBLT_COLORFILL|DDBLT_WAIT,&ddbltfx);

}


void DirectXManagerObject::Flip() {
	// this function handles the flipping of the primary and
	// secondary surfaces
	
	lpddsprimary->Flip(NULL,DDFLIP_WAIT);
	Sleep(40);
}

void DirectXManagerObject::destroySurfaces() {
	// this function releases both the primary and secondary
	// surfaces, used before shutting down program

	if (lpddsecondary)
		lpddsecondary->Release();
	if (lpddsprimary)
		lpddsprimary->Release();

}


void DirectXManagerObject::shutdownddraw() {
	// this program shuts down the directdraw object
	// used before shutting down program

	if ( lpdirectdrawobj )
		lpdirectdrawobj->Release();

}

int DirectXManagerObject::createdinputKeyboard(HINSTANCE main_instance_handle,
											   HWND main_window_handle) {

	// this function sets up a DirectDraw object, and then attaches
	// the standard keyboard as an input device.  Will be changed
	// in future programs.

	
	// first create the direct input object
	if(DirectInputCreate(main_instance_handle,DIRECTINPUT_VERSION,
						&lpdirectinputobj,NULL) != DI_OK)
		return(0);

	// create a keyboard device
	if (lpdirectinputobj->CreateDevice(GUID_SysKeyboard,&lpdinputkey,
										NULL) != DI_OK)
		return(0);
	
	// set cooperation level
	if (lpdinputkey->SetCooperativeLevel(main_window_handle,
							DISCL_BACKGROUND|DISCL_NONEXCLUSIVE) != DI_OK)
		return(0);

	// set data format
	if (lpdinputkey->SetDataFormat(&c_dfDIKeyboard)!=DI_OK)
		return(0);

	//--------------------------------------------//


	// acquire the keyboard for use.
	if (lpdinputkey->Acquire() != DI_OK)
		return(0); 

  
	return(1);

}

int DirectXManagerObject::createdirectsound(HWND window_handle) {
	// this program creates a DirectSound object used to playback
	// sounds
	

	// create the Direct Sound object
	if (DirectSoundCreate(NULL,&lpdirectsoundobj, NULL) != DS_OK) 
		return(0);	

	// set the cooperation level to normal
	if (lpdirectsoundobj->SetCooperativeLevel(window_handle,DSSCL_NORMAL) != DS_OK)
		return(0);
	
	return(1);
}

void DirectXManagerObject::shutdowndInputKey() {
	// this function releases the keyboard and shuts down the
	// DirectInput object.  Used before quitting a program

	lpdinputkey->Unacquire();
	lpdinputkey->Release();

	lpdirectinputobj->Release();

}

void DirectXManagerObject::shutdowndsound() {
	// this function shuts down the DirectSound object

	lpdirectsoundobj->Release();
}

		
bool DirectXManagerObject::startupDirectX(HINSTANCE main_instance, HWND main_window_handle) {
	// this method sets up the DirectDraw, DirectInput, and 
	// DirectSound objects, and creates both the primary and 
	// secondary surfaces and sets up the keyboard for input.

	// create a DirectDraw object to allow displaying of 
	// 2D image.  Make it fullscreen.
	if ( !createdirectdraw(SCREEN_WIDTH,SCREEN_HEIGHT,SCREEN_BPP,
		                   main_window_handle))
		return false;
	
	// create the secondary and primary surfaces
	if ( !createsurfaces())
		return false;

	// create a DirectSound object for playing sound
	if ( !createdirectsound(main_window_handle))
		return false;

	// create DirectInput object and initialize use of keyboard
	if ( !createdinputKeyboard(main_instance,main_window_handle) )
		return false;

	return true;

}

bool DirectXManagerObject::shutdownDirectX() {
	// this method deallocates all DirectX variables
	// should be last used function before shutting down
	// program.

	destroySurfaces();

	shutdownddraw();
	shutdowndInputKey();
	shutdowndsound();

	return true;

}


