// objects.cpp

// Updated by Lucas Keyes
// January 17, 2002

#include "objects.h"
#include "timer.h"    // game timer

extern CTimer Timer;

Game_Object::Game_Object() {    // constructor
    gX = gY = g_Xspeed = g_Yspeed = 0;
    g_Sprite = NULL; g_lastXMoveTime = 0;
}





void Game_Object::draw(int frame, LPDIRECTDRAWSURFACE surface) {  // draw
    
    g_Sprite->draw(frame,gX,gY,surface);

}

void Game_Object::create(int x, int y, int xspeed, int yspeed,
                         SpriteImg_Object *sprite) {

    g_lastXMoveTime = 0;    // time
    gX = x; gY = y;         // location
    g_Xspeed = xspeed; g_Yspeed = yspeed;  // speed
    g_Sprite = sprite;     // image
}

void Game_Object::accelerate(int xdelta, int ydelta) {

    // change speed
    g_Xspeed+=xdelta; g_Xspeed+=ydelta;
}

void Game_Object::move() {  // move object
   
    const int XSCALE = 8;   // to scale back horizontal motion
    const int MARGIN = 100; // margin on outside of page
    int xdelta;             // change in position
    int time = Timer.time(); // current time

	// horizontal motion
	int tfactor = time-g_lastXMoveTime;		// time since last move
	xdelta = (g_Xspeed*tfactor)/XSCALE;		// x distance moved
	gX+=xdelta;		// x motion
	
	if (gX<-MARGIN)gX=SCREEN_WIDTH+MARGIN;	// wrap left
	if (xdelta||g_Xspeed==0)	// record time of move
		g_lastXMoveTime=time;
    
	if (gX>SCREEN_WIDTH+MARGIN)gX=-MARGIN;	// wrap right
} // end move


